
-- The different types of objects for permissions are "securable classes"
-- A common securable class is object or "schema object"
-- Sys.dm_audit_class_type_map is a comprehensive mapping between 
-- metadata object types and securable classes.

-- Report not only what happened but what audit configurations may have 
-- been enabled to cause the event to fire. 
-- Sometimes the sys.dm_audit_actions table reports as class_type_desc 
-- the metadata object type and sometimes the securable class

-- Case where the sys.dm_audit_actions reports securable class and the 
-- securable class is not the same as the metadata object type. 
SELECT event_time, sequence_number, a.action_id, a.class_type, 
       b.securable_class_desc, b.class_type_desc, c.name  
FROM fn_get_audit_file( N'c:\temp\*.sqlaudit', null, null ) AS a 
inner join sys.dm_audit_class_type_map AS b 
    on (a.class_type = b.class_type) 
inner join sys.dm_audit_actions c on (c.action_id = a.action_id and 
    c.class_desc = b.securable_class_desc) 
where not exists(select * from sys.dm_audit_actions d where d.action_id = 
    a.action_id and d.class_desc = b.class_type_desc) 
union 
-- Case where the sys.dm_audit_actions reports class_type_desc, or 
-- metadata object type. 
SELECT event_time, sequence_number, a.action_id, a.class_type, 
       b.securable_class_desc, b.class_type_desc, c.name 
FROM fn_get_audit_file( N'c:\temp\*.sqlaudit', null, null ) AS a 
inner join sys.dm_audit_class_type_map AS b 
    on (a.class_type = b.class_type) 
inner join sys.dm_audit_actions c on (c.action_id = a.action_id and 
    c.class_desc = b.class_type_desc) 
