/*
	TDE Example
*/
use master
go
create database TDE
go
use TDE
go
create table dbo.t1(a varchar(100))
insert t1(a) 
values ('This is some plain text'),('This is some more plain text')
go
use master
go
backup database TDE to disk = 'c:\TDEUnencrypted.bak' with init
go
CREATE MASTER KEY ENCRYPTION BY PASSWORD = 'sd092735kjn$&adsg' 
GO 
BACKUP MASTER KEY TO FILE = 'c:\masterkey'  
    ENCRYPTION BY PASSWORD = 'sd092735kjn$&adsg' 
GO  
use master
go
CREATE CERTIFICATE DEKCert WITH SUBJECT = 'DEK Certificate'
go
BACKUP CERTIFICATE DEKCert TO FILE = 'c:\DEKCert'
    WITH PRIVATE KEY ( FILE = 'c:\DEKCertPrivKey' , 
    ENCRYPTION BY PASSWORD = 'sd092rhhh4735kjn$&adsg' )
GO

USE TDE
GO
CREATE DATABASE ENCRYPTION KEY
WITH ALGORITHM = AES_128
ENCRYPTION BY SERVER CERTIFICATE DEKCert
GO
ALTER DATABASE TDE SET ENCRYPTION ON
GO

SELECT * FROM sys.dm_database_encryption_keys
GO

select db_name(e.database_id) as DatabaseName,
e.database_id,
e.encryption_state,
case e.encryption_state
when 0 then 'No database encryption key present, no encryption'
when 1 then 'Unencrypted'
when 2 then 'Encryption in progress'
when 3 then 'Encrypted'
when 4 then 'Key change in progress'
when 5 then 'Decryption in progress'
end as encryption_state_desc,
c.name,
e.percent_complete
from sys.dm_database_encryption_keys as e
left join master.sys.certificates as c
on e.encryptor_thumbprint = c.thumbprint
go

-- 1.First we will try and restore the TDE database to another instance without first 
-- creating the server certificate. This fails because the server certificate used 
-- to secure the database encryption key cannot be found on the new instance. 

-- backup database if not already done 
BACKUP DATABASE TDE  
TO DISK = 'c:\TDE_Encrypted.bak' WITH init 
GO 

-- try and restore to a different instance 
RESTORE DATABASE TDE  
FROM DISK = 'c:\TDE_Encrypted.bak' 
WITH move 'TDE' TO 'c:\TDE.mdf', 
move 'TDE_log' TO 'c:\TDE.ldf'
GO   

/*
Msg 33111, Level 16, State 3, Line 1
Cannot find server certificate with thumbprint '0xCA741797B81ED8D1305EAE65A747BA51E1DAB80D'.
Msg 3013, Level 16, State 1, Line 1
RESTORE DATABASE is terminating abnormally.
*/

-- 2.On our new instance, we need to create the server certificate used to secure 
-- our database encryption key for the TDE database (DEKCert). 
-- We do this by creating a certificate from the backups we created in Step 4 
-- of Part I (if you don't have the backups simply run the backup certificate part again on the original instance). We need to create the database master key for the master database as in Part I if it doesn't exist before we can create the certificate.

USE master 
GO 
-- create database master key if it doesn't exist 
CREATE MASTER KEY ENCRYPTION BY PASSWORD = 'sdfsdfysd092735kjn$&adsg' 
GO 
--create TDE certificate from backup 
CREATE CERTIFICATE DEKCert  
FROM FILE = 'c:\DEKCert' 
WITH PRIVATE KEY (FILE = 'c:\DEKCertPrivKey', 
DECRYPTION BY PASSWORD = 'sd092rhhh4735kjn$&adsg') 
GO   

-- 3.Now that the server certificate used to encrypt the database encryption key 
-- for the TDE database is present on the new instance, we can successfully restore 
-- the TDE database.

RESTORE DATABASE TDE  
FROM DISK = 'c:\TDE_Encrypted.bak' 
WITH move 'TDE' TO 'c:\TDE.mdf', 
move 'TDE_log' TO 'c:\TDE.ldf', 
recovery 
GO  

-- Just to make sure, you can query the test table to make sure you can see the data

SELECT * FROM TDE.dbo.t1   

